###########################################################################
#
#   Copyright (c) 2000,2001 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000,2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.14 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import OFS.Folder
import calendar_year
import product_base_item
import base_item
import time
import calendar
import todo_category_folder
import todo_category
import utils
import DateTime
import string

calendar_folder_add_redirect = HTMLFile('calendar_folder_add_redirect', globals())
manage_add_calendar_folder_form = HTMLFile('calendar_folder_add', globals())


def manage_add_calendar_folder(self, id, title,
			install_default_html='',
			install_default_theme='',
			install_default_id_generator='',
			REQUEST=None):
	"""Add an instance of the calendar_folder class."""

	self._setObject(id, calendar_folder(id, title,
				install_default_html=install_default_html,
				install_default_theme=install_default_theme,
				install_default_id_generator=install_default_id_generator))

	if REQUEST is not None:

		REQUEST.RESPONSE.redirect(self.absolute_url())

	else:

		return self[id]


class calendar_folder(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	product_base_item.product_base_item,
	base_item.base_item
	):
	"""Calendar used for scheduling events, todos and such."""

	use_local_navigation = 0

	meta_type = 'CalendarFolder'
	icon = 'misc_/CalendarFolder/Folder_icon.gif'
	default_catalog = 'Catalog'

	_properties = (
		{'id':'title', 'type':'string'},
		{'id':'snooze', 'type':'int', 'select_widget':'snooze_values'},
		)

	meta_types = (
		{'name':'TodoCategoryFolder', 'action':'manage_add_todo_category_folder_form'},
		{'name':'CalendarYear', 'action':'manage_add_calendar_year_form'},
		)

	manage_add_calendar_year_form = calendar_year.manage_add_calendar_year_form
	manage_add_calendar_year = calendar_year.manage_add_calendar_year
	calendar_year_add_redirect = calendar_year.calendar_year_add_redirect

	manage_add_todo_category_folder_form = todo_category_folder.manage_add_todo_category_folder_form
	manage_add_todo_category_folder = todo_category_folder.manage_add_todo_category_folder

	# How much leeway (seconds) we give when checking for events that may conflict.

	looseness = 120

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('Add CalendarYear',
		 ('manage_add_calendar_year', 'manage_add_calendar_year_form')),
		('View CalendarFolders',
		 ('', 'get_years', 'get_ymd', 'get_date', 'display_current_month',
			'get_today', 'display_current_day', 'display_next_day',
			'get_day', 'check_timespan', 'get_unique_id',
			'get_calendar')),
		('Manage Calendar',
			('calendar_folder_add_event', 'manage_add_calendar_folder_event'))
		)

	alternate_date_mappings = [
			{'name':'', 'title':'Select date'},
			{'name':0, 'title':'Today'},
			{'name':1, 'title':'Tomorrow'},
			{'name':7, 'title':'Today + 1 week'},
			{'name':31, 'title':'Today + 1 month'}
		]

	snooze_values = {
		300:'5 minutes',
		600:'10 minutes',
		900:'15 minutes',
		1800:'30 minutes',
		3600:'1 hour'
		}

	index_html = HTMLFile('calendar_folder_index', globals())
	calendar_folder_add_event = HTMLFile('calendar_folder_add_event',
						globals())
	calendar_folder_add_todo = HTMLFile('calendar_folder_add_todo', globals())
	global_todo_listing = HTMLFile('global_todo_listing', globals())
	global_event_listing = HTMLFile('global_event_listing', globals())
	display_timespan = HTMLFile('calendar_folder_display_timespan', globals())

	product_name = 'CalendarFolder'


	def get_specified_todos(self, REQUEST):
		"""Returns a list of todos selected via REQUEST."""

		# FIXME

		return self.get_todos()


	def get_specified_events(self, REQUEST):
		"""Returns a list of events selected via REQUEST."""

		# FIXME

		return self.get_events()


	def get_todo_categories(self):
		"""Returns a list of all the todo categories."""

		categories = []

		for todo_category_folder in self.objectValues('TodoCategoryFolder'):

			categories.extend(todo_category_folder.objectValues('TodoCategory'))

		return categories


	def get_todo_category_folders(self):
		"""Returns a list of all the todo category folders."""

		return self.objectValues('TodoCategoryFolder')


	def get_default_todo_category(self):
		"""Returns the default todo category."""

		return self.get_todo_category_folders()[0].get_default_todo_category()


	def get_todos(self):
		"""Returns a list of all the todos."""

		return self.ZopeFind(self, obj_metatypes=['Todo'], search_sub=1)


	def get_events(self):
		"""Returns a list of all events."""

		return self.ZopeFind(self, obj_metatypes=['CalendarEvent'], search_sub=1)


	def render_alternate_date_widget(self):
		"""Returns a widget helpful for selecting dates."""

		select_widget = """<select name="alternate_date_mapping">"""

		for option in self.alternate_date_mappings:

			select_widget = select_widget + """<option value="%s">%s</option>""" % \
						(option['name'], option['title'])

		return select_widget + "</select>"


	def render_category_widget(self, category=None):
		"""Returns a select widget."""

		select_widget = """<select name="category">"""

		for category in self.get_todo_categories():

			if category and category.id == category:

				selected = " selected"

			else:

				selected = ""

			select_widget = select_widget + """<option value="%s"%s>%s</option>""" % \
						(category.id, selected, category.title)

		return select_widget + "</select>"


	def render_year_widget(self, year=None, name=''):
		"""Returns a year widget."""

		if name:

			name = name + '_year'

		else:

			name = 'year'

		current_year = self.get_ymd()[0]

		return utils.make_range_select(name, current_year, current_year + 10,
				pad=1, selected=year, integer=None)


	def render_month_widget(self, month=None, name=''):
		"""Returns a month widget."""

		if name:

			name = name + '_month'

		else:

			name = 'month'

		if month is not None:

			selected_month = month

		else:

			selected_month = self.get_ymd()[1]

		return utils.make_range_select(name, 1, 13, pad=1,
				selected=selected_month, integer=None)


	def render_day_widget(self, day=None, name=''):
		"""Returns a day widget."""

		if name:

			name = name + '_day'

		else:

			name = 'day'

		if day is not None:

			selected_day = day

		else:

			selected_day = self.get_ymd()[2]

		return utils.make_range_select(name, 1, 32, pad=1,
				selected=selected_day, integer=None)


	def get_years(self):
		"""Returns all year objects contained, sorted ascendingly."""

		calendar_years = self.objectValues('CalendarYear')

		calendar_years.sort(lambda x,y: cmp(int(x.id), int(y.id)))

		return calendar_years


	def get_ymd(self):
		"""Returns a tuple of the current year, month and date."""

		return time.localtime(time.time())[:3]


	def get_date(self, year, month, day):
		"""Returns the specified day, by date."""

		# We need to add some handling on invalid dates
		# here, FIXME

		year = utils.pad_integer(year)
		month = utils.pad_integer(month)
		day = utils.pad_integer(day)

		year = self.get_year(year)

		return year[month][day]


	def get_current_month(self):
		"""Gets the current month."""

		year = utils.pad_integer(self.get_date()[0])
		month = utils.pad_integer(self.get_date()[1])

		return self[year][month]


	def get_current_day(self):
		"""Renders the current day."""

		return self.get_day()


	def get_next_day(self):
		"""Returns the next day."""

		return self.get_current_day().get_next_day()


	def get_previous_day(self):
		"""Returns the previous day."""

		return self.get_today().get_previous_day()


	def get_day(self, offset=0):
		"""Returns the day depending on the offset."""

		current_date = DateTime.DateTime()

		new_date = DateTime.DateTime(int(current_date) + (86400 * offset))

		year = utils.pad_integer(new_date.year())
		month = utils.pad_integer(new_date.month())
		day = utils.pad_integer(new_date.day())

		return self[year][month][day]		


	def check_timespan(self, year, month, day,
				start, end, looseness=120):
		"""Checks if given timespan is available.

		year is an integer specifying the year,

		month is an integer specifying the month,

		day is an integer specifying the day,

		start is an integer (in the context of the three variables
		above) indicating the starting time.

		end is the same as above, only indicating the end
		instead of the start

		loseness is an integer specifying the looseness of
		the possible matching events, in seconds.
		"""

		conflicting_events = []

		if len(month) < 2:

			month = '0' + str(month)

		if len(day) < 2:

			day = '0' + day

		if type(year) is type('') and \
		   type(month) is type('') and \
		   type(day) is type(''):

			pass

		else:

			month = str(month)
			day = str(day)
			year = str(year)

		events = self[year][month][day].get_events()

		# We remove completed and cancelled events.

		events = filter(lambda x: x.status == 1, events)

		for event in events:

			if start >= event.start and start + looseness <= event.end:

				conflicting_events.append(event)

			elif end >= event.start + self.looseness and end <= event.end:

				conflicting_events.append(event)

			elif start <= event.start and end >= event.end:

				conflicting_events.append(event)


		return conflicting_events


	def manage_add_calendar_folder_event(self, year, month, day,
						REQUEST, RESPONSE):
		"""Adds a calendar event, 'from the root'."""

		year = utils.pad_integer(year)
		month = utils.pad_integer(month)
		day = utils.pad_integer(day)

		day = self[year][month][day]

		day.manage_add_calendar_event(
			REQUEST=REQUEST, RESPONSE=RESPONSE)

		RESPONSE.redirect(day.absolute_url())


	def manage_add_calendar_folder_todo(self, year, month, day,
						alternate_date_mapping,
						REQUEST, RESPONSE):
		"""Adds a calendar todo, 'from the root'."""

		if alternate_date_mapping:

			datetime = DateTime.DateTime()
			datetime = datetime + alternate_date_mapping

			year, month, day = string.split(datetime.Date(), '/')

		else:

			year = utils.pad_integer(REQUEST['year'])
			month = utils.pad_integer(REQUEST['month'])
			day = utils.pad_integer(REQUEST['day'])

		"""
		due_year = utils.pad_integer(REQUEST['due_year'])
		due_month = utils.pad_integer(REQUEST['due_month'])
		due_day = utils.pad_integer(REQUEST['due_day'])
		"""

		due_year, due_month, due_day = (None, None, None)

		day = self[year][month][day]

		day.manage_add_todo(REQUEST['title'],
					REQUEST['body'],
					REQUEST['priority'],
					REQUEST['category'],
					due_year=due_year,
					due_month=due_month,
					due_day=due_day)

		RESPONSE.redirect(day.absolute_url())	


	def move_todos(self):
		"""Moves any unfinished todos from the previous day to the next."""

		previous_day = self.get_previous_day()
		today = self.get_today()

		for todo in previous_day.get_todos_pending():

			today.paste(todo.cut())


	def __init__(self, id, title='',
			install_default_html='',
			install_default_theme='',
			install_default_id_generator=''):
		"""Initialize the class instance."""

		self.id = id
		self.title = title
		self.snooze = 300

		calendar_year.manage_add_calendar_year(self,
			str(self.get_ymd()[0]), '')

		todo_category_folder_id = todo_category_folder.manage_add_todo_category_folder(self,
						id='TodoCategories', title='Todo categories')

		todo_category.manage_add_todo_category(self[todo_category_folder_id],
								id='-2', title='Business')
		todo_category.manage_add_todo_category(self[todo_category_folder_id],
								id='-3', title='Personal')
		todo_category.manage_add_todo_category(self[todo_category_folder_id],
								id='-4', title='Recreational')

		product_base_item.utils.default_installation(
			self,
			install_default_html=install_default_html,
			install_default_theme=install_default_theme,
			install_default_id_generator=install_default_id_generator
				)


	def get_calendar(self):
		"""Returns self."""

		return self


default__class_init__(calendar_folder)

